/******************************************************
LCD Driver for Hitachi HD44780 compatible chip based
LCD Displays. Support only 2 lines display, but
you can simply modify it to work with 1 line displays.

Tested with 16x2 character display, and 4bits wiring.
Wiring described below.

// D0: RS
// D1: R/W  0=W, 1=R
// D2: ENABLE
// D3: DB0
// D4: DB1
// D5: DB2
// D6: DB3
// D7: NOT USED

Author:  Rudolf Szendrei
License: GNU/GPL
E-Mail:  szr@inf.elte.hu

******************************************************/

#ifndef HD44780_DRIVER_H
#define HD44780_DRIVER_H

#define LCD_INSTRUCTION 0
#define LCD_DATA        1
#define LCD_WRITE       0
#define LCD_READ        1
#define WAIT_TIME       10

#include <asm/io.h>
#include <linux/delay.h>

char str[] = {"Testing 1,2,3                           "
	      "It' Works !                             "};

void lcd_send_4_bits(char, char, int);
void lcd_send_byte(char, char, int);
void lcd_clear(void);
void lcd_putc(char);
void lcd_write(const char*, int);
void lcd_gohome(void);
void lcd_gotoxy(int, int);
void lcd_setcharpattern(char, const char*);
void lcd_init(void);

/* ---------------------------------------------------- */

void lcd_send_4_bits(char c, char rs, int Wait)
{
    c = c << 4;
    outb((c | 0x04) | rs, LPT_PORT); // ENABLE
    outb((c & 0xf0) | rs, LPT_PORT); // DISABLE

    mdelay(Wait);
}

void lcd_send_byte(char c, char rs, int Wait)
{
    lcd_send_4_bits( (c >> 4), rs, Wait );
    lcd_send_4_bits( (c & 15), rs, Wait );
}

void lcd_clear()
{
    lcd_send_byte( 0x01, LCD_INSTRUCTION, 5 );
}


void lcd_putc(char c)
{
    lcd_send_byte( c, LCD_DATA, 1 );
}

void lcd_write(const char* str, int len)
{
    int i;
    for( i = 0; i < len; ++i) 
	lcd_putc(str[i]);
}

void lcd_gohome()
{
    lcd_send_byte( 0x02, LCD_INSTRUCTION, 5 );
}

void lcd_gotoxy(int x, int y)
{
    char c = 0x80 | (x & 0x3f) | (y << 6);

    lcd_send_byte( c, LCD_INSTRUCTION, 1);
}

void lcd_setcharpattern(char c_pos, const char* pattern)
{
    char c = 0x40 | ((c_pos & 0x3f) << 3);

    // SELECT CGRAM ADDRESS
    lcd_send_byte( c, LCD_INSTRUCTION, 1);

    // UPLOAD FONT PATTERN
    lcd_send_byte( pattern[0], LCD_DATA, 1 );
    lcd_send_byte( pattern[1], LCD_DATA, 1 );
    lcd_send_byte( pattern[2], LCD_DATA, 1 );
    lcd_send_byte( pattern[3], LCD_DATA, 1 );
    lcd_send_byte( pattern[4], LCD_DATA, 1 );
    lcd_send_byte( pattern[5], LCD_DATA, 1 );
    lcd_send_byte( pattern[6], LCD_DATA, 1 );
    lcd_send_byte( pattern[7], LCD_DATA, 1 );
}

void lcd_init(void)
{
    int  i;
    char init[5];

    init[0] = 0x03; // INIT
    init[1] = 0x02; 
    init[2] = 0x08; 
    init[3] = 0x0C; // DISPLAY ON
    init[4] = 0x06; // ENTRY MODE SET

    outb(0, LPT_PORT);

    // FUNCTION SET 4 BIT INIT
    for (i = 0; i < 3; i++) lcd_send_4_bits( init[0], LCD_INSTRUCTION, 20 );

    // FUNCTION SET 4 BIT INIT
    for (i = 0; i < 2; i++) lcd_send_4_bits( init[1], LCD_INSTRUCTION, 20 );

    // LINES 2, 5x8 FONT
    lcd_send_4_bits( init[2], LCD_INSTRUCTION, 20 );

    // DISPLAY ON
    lcd_send_byte( init[3], LCD_INSTRUCTION, 10 );

    // DISPLAY CLEAR
    lcd_clear();

    // ENTRY MODE SET
    lcd_send_byte( init[4], LCD_INSTRUCTION, 5 );
/*
    char pattern_temp[] = { 0x0F, 0x0C, 0x0F, 0x0C, 0x0E, 0x11, 0x1F, 0x0E };
    lcd_setcharpattern(0, (char*)pattern_temp);

    char pattern_accu[] = { 0x08, 0x1C, 0x0B, 0x00, 0x1B, 0x1F, 0x11, 0x1F };
    lcd_setcharpattern(1, (char*)pattern_accu);

    char pattern_rpm[]  = { 0x00, 0x0D, 0x1F, 0x17, 0x10, 0x1B, 0x0E, 0x00 };
    lcd_setcharpattern(2, (char*)pattern_rpm);

    char pattern_sep[]  = { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 };
    lcd_setcharpattern(3, (char*)pattern_sep);
*/
}

#endif
